The Linux Kernel Mailing List
 help / color / mirror / Atom feed
From: Petr Mladek <pmladek@suse.com>
To: Aaron Tomlin <atomlin@atomlin.com>
Cc: akpm@linux-foundation.org, lance.yang@linux.dev,
	mhiramat@kernel.org, gregkh@linuxfoundation.org, sean@ashe.io,
	Joel Granados <joel.granados@kernel.org>,
	linux-kernel@vger.kernel.org
Subject: Re: [PATCH v3 2/2] hung_task: Enable runtime reset of hung_task_detect_count
Date: Wed, 17 Dec 2025 13:48:07 +0100	[thread overview]
Message-ID: <aUKmh0Gs8YH_iLbC@pathway.suse.cz> (raw)
In-Reply-To: <20251216030036.1822217-3-atomlin@atomlin.com>

Adding Joel into Cc. He is improving the sysctl API...

On Mon 2025-12-15 22:00:36, Aaron Tomlin wrote:
> Introduce support for writing to /proc/sys/kernel/hung_task_detect_count.
> 
> Writing any value to this file atomically resets the counter of detected
> hung tasks to zero. This grants system administrators the ability to clear
> the cumulative diagnostic history after resolving an incident, simplifying
> monitoring without requiring a system restart.
> 
> --- a/Documentation/admin-guide/sysctl/kernel.rst
> +++ b/Documentation/admin-guide/sysctl/kernel.rst
> @@ -418,7 +418,7 @@ hung_task_detect_count
>  ======================
>  
>  Indicates the total number of tasks that have been detected as hung since
> -the system boot.
> +the system boot. The counter can be reset to zero when written to.
>  
>  This file shows up if ``CONFIG_DETECT_HUNG_TASK`` is enabled.
>  
> diff --git a/kernel/hung_task.c b/kernel/hung_task.c
> index 5902573200c0..01ce46a107b0 100644
> --- a/kernel/hung_task.c
> +++ b/kernel/hung_task.c
> @@ -375,6 +375,31 @@ static long hung_timeout_jiffies(unsigned long last_checked,
>  }
>  
>  #ifdef CONFIG_SYSCTL
> +
> +/**
> + * proc_dohung_task_detect_count - proc handler for hung_task_detect_count
> + * @table: Pointer to the struct ctl_table definition for this proc entry
> + * @write: Flag indicating the operation
> + * @buffer: User space buffer for data transfer
> + * @lenp: Pointer to the length of the data being transferred
> + * @ppos: Pointer to the current file offset
> + *
> + * This handler is used for reading the current hung task detection count
> + * and for resetting it to zero when a write operation is performed.
> + * Returns 0 on success or a negative error code on failure.
> + */
> +static int proc_dohung_task_detect_count(const struct ctl_table *table, int write,
> +					 void *buffer, size_t *lenp, loff_t *ppos)
> +{
> +	if (!write)
> +		return proc_doulongvec_minmax(table, write, buffer, lenp, ppos);

There have been some changes in the sysctl API recently, see
https://lore.kernel.org/lkml/20251016-jag-sysctl_conv-v2-0-a2f16529acc4@kernel.org/

They are backward compatible, so the above code works. But it would be
nice to make it up-to-date, namely:

  + Replace "write" with "dir"
  + Use SYSCTL_USER_TO_KERN(dir) instead of (!write)


> +	WRITE_ONCE(sysctl_hung_task_detect_count, 0);

I might be too conservative. But it looks weird to allow clearing the
value by any write. It would be better to return -EINVAL for non-zero
values. This would require using a copy of struct ctl_table and read
the value into a temporary variable.

> +	*ppos += *lenp;
> +
> +	return 0;
> +}

I have played with the code. The diff on top of this patch would
look like:

diff --git a/kernel/hung_task.c b/kernel/hung_task.c
index 01ce46a107b0..ebb3dfd0b148 100644
--- a/kernel/hung_task.c
+++ b/kernel/hung_task.c
@@ -379,7 +379,7 @@ static long hung_timeout_jiffies(unsigned long last_checked,
 /**
  * proc_dohung_task_detect_count - proc handler for hung_task_detect_count
  * @table: Pointer to the struct ctl_table definition for this proc entry
  * @write: Flag indicating the operation
  * @buffer: User space buffer for data transfer
  * @lenp: Pointer to the length of the data being transferred
  * @ppos: Pointer to the current file offset
@@ -388,16 +388,29 @@ static long hung_timeout_jiffies(unsigned long last_checked,
  * and for resetting it to zero when a write operation is performed.
  * Returns 0 on success or a negative error code on failure.
  */
-static int proc_dohung_task_detect_count(const struct ctl_table *table, int write,
+static int proc_dohung_task_detect_count(const struct ctl_table *table, int dir,
 					 void *buffer, size_t *lenp, loff_t *ppos)
 {
-	if (!write)
-		return proc_doulongvec_minmax(table, write, buffer, lenp, ppos);
+	unsigned long detect_count = READ_ONCE(sysctl_hung_task_detect_count);
+	struct ctl_table t;
+	int err;
 
-	WRITE_ONCE(sysctl_hung_task_detect_count, 0);
-	*ppos += *lenp;
+	t = *table;
+	t.data = &detect_count;
 
-	return 0;
+	err = proc_doulongvec_minmax(&t, dir, buffer, lenp, ppos);
+	if (err < 0)
+		return err;
+
+	if (SYSCTL_USER_TO_KERN(dir)) {
+		/* The only valid value for clearing is zero. */
+		if (detect_count)
+			return -EINVAL;
+
+		WRITE_ONCE(sysctl_hung_task_detect_count, 0);
+	}
+
+	return err;
 }
 
 /*
@@ -379,7 +379,7 @@ static long hung_timeout_jiffies(unsigned long last_checked,
  * proc_dohung_task_detect_count - proc handler for hung_task_detect_count
  * @table: Pointer to the struct ctl_table definition for this proc entry
- * @write: Flag indicating the operation
+ * @dir: Flag indicating the operation
  * @buffer: User space buffer for data transfer
  * @lenp: Pointer to the length of the data being transferred


Best Regards,
Petr

  parent reply	other threads:[~2025-12-17 12:48 UTC|newest]

Thread overview: 17+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2025-12-16  3:00 [PATCH v3 0/2] hung_task: Provide runtime reset interface for hung task detector Aaron Tomlin
2025-12-16  3:00 ` [PATCH v3 1/2] hung_task: Introduce helper for hung task warning Aaron Tomlin
2025-12-17  9:39   ` Petr Mladek
2025-12-21 21:52     ` Aaron Tomlin
2025-12-16  3:00 ` [PATCH v3 2/2] hung_task: Enable runtime reset of hung_task_detect_count Aaron Tomlin
2025-12-17  7:31   ` Lance Yang
2025-12-17 13:09     ` Petr Mladek
2025-12-17 13:36       ` Lance Yang
2025-12-19  3:09       ` Aaron Tomlin
2025-12-19 12:03         ` Petr Mladek
2025-12-19 14:15           ` Lance Yang
2025-12-21 21:00             ` Aaron Tomlin
2025-12-17 12:48   ` Petr Mladek [this message]
2025-12-17 13:21     ` Lance Yang
2025-12-18  9:08       ` Joel Granados
2025-12-19 14:18         ` Lance Yang
2025-12-21 21:26         ` Aaron Tomlin

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=aUKmh0Gs8YH_iLbC@pathway.suse.cz \
    --to=pmladek@suse.com \
    --cc=akpm@linux-foundation.org \
    --cc=atomlin@atomlin.com \
    --cc=gregkh@linuxfoundation.org \
    --cc=joel.granados@kernel.org \
    --cc=lance.yang@linux.dev \
    --cc=linux-kernel@vger.kernel.org \
    --cc=mhiramat@kernel.org \
    --cc=sean@ashe.io \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox